package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.PlanDetailDTO;
import com.diagbot.dto.PlanInfoDefaultDTO;
import com.diagbot.entity.Plan;
import com.diagbot.entity.PlanDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.PlanDefaultVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-11-20 9:32
 */
@Component
public class PlanDefaultFacade {
    @Autowired
    PlanDetailFacade planDetailFacade;
    @Autowired
    PlanFacade planFacade;
    @Autowired
    DictionaryFacade dictionaryFacade;

    public Boolean addDefaultPlans(PlanDefaultVO planDefaultVO) {
        boolean res = false;
        Date now = DateUtil.now();
        List<DictionaryInfoDTO> listByGroupType = dictionaryFacade.getListByGroupType(60);
        List<String> planCode = listByGroupType.stream().map(ac -> ac.getVal()).collect(Collectors.toList());
       /* List<String> planCode = Arrays.asList("plan_default", "patientia", "info", "chief", "symptom", "other", "vital"
                , "lis", "pacs", "diag", "doc_advice", "reference_lis", "reference_pacs", "order_rational",
                "medical_advice", "medical_record");*/

        //先删除医院默认的方案，
        if (planDefaultVO.getHospitalId() == -1) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "朗通医院不能一键添加！");
        }
        QueryWrapper<Plan> planFandList = new QueryWrapper<>();
        planFandList.eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("plan_code", planCode)
                .eq("hospital_id", planDefaultVO.getHospitalId());
        List<Plan> planList = planFacade.list(planFandList);
        if (ListUtil.isNotEmpty(planList)) {
            List<Long> planIdcancel = planList.stream().map(ac -> ac.getId()).collect(Collectors.toList());
            UpdateWrapper<PlanDetail> planDetailcancel = new UpdateWrapper<>();
            planDetailcancel.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", planDefaultVO.getHospitalId())
                    .in("plan_id", planIdcancel);

            planDetailFacade.remove(planDetailcancel);
            planFacade.remove(planFandList);
        }

        //查询默认方案
        for (String str : planCode) {
            PlanInfoDefaultDTO data = new PlanInfoDefaultDTO();
            QueryWrapper<Plan> planFand = new QueryWrapper<>();
            planFand.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("plan_code", str)
                    .eq("hospital_id", -1L);
            Plan plan = planFacade.getOne(planFand, false);
            BeanUtil.copyProperties(plan, data);
            List<Long> planId = new ArrayList<>();
            planId.add(plan.getId());
            HospitalSetVO hospitalSetVO = new HospitalSetVO();
            hospitalSetVO.setHospitalId(-1L);
            hospitalSetVO.setPlanId(planId);
            List<PlanDetailDTO> sysSetInfoData = planDetailFacade.getSysSetInfoData(hospitalSetVO);
            data.setPlanDetailDefault(sysSetInfoData);
            //先效验改医院下是否存在该方案
            int countTow = planFacade.count(new QueryWrapper<Plan>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", planDefaultVO.getHospitalId())
                    .eq("plan_code", str));
            if (countTow > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, str + "该方案编码已存在");
            }
            //保存方案配置
            if (plan != null) {
                Plan planDefault = new Plan();
                planDefault.setHospitalId(planDefaultVO.getHospitalId());
                planDefault.setPlanCode(plan.getPlanCode());
                planDefault.setPlanName(plan.getPlanName());
                planDefault.setPlanStatus(1);
                planDefault.setRemark("0");
                planDefault.setModifier(UserUtils.getCurrentPrincipleID());
                planDefault.setCreator(UserUtils.getCurrentPrincipleID());
                planDefault.setGmtModified(now);
                planDefault.setGmtCreate(now);
                res = planFacade.save(planDefault);
                if (res) {
                    Long planIdDetail = planDefault.getId();
                    List<PlanDetailDTO> PlanDetailDefault = new ArrayList<>();
                    for (PlanDetailDTO dataDetail : sysSetInfoData) {
                        //先保存父类
                        PlanDetail planDetailParent = new PlanDetail();
                        planDetailParent.setParentId(-1L);
                        planDetailParent.setPlanId(planIdDetail);
                        planDetailParent.setName(dataDetail.getName());
                        planDetailParent.setCode(dataDetail.getCode());
                        planDetailParent.setHospitalId(planDefaultVO.getHospitalId());
                        planDetailParent.setStatus(dataDetail.getStatus());
                        planDetailParent.setOrderNo(dataDetail.getOrderNo());
                        planDetailParent.setNumber(dataDetail.getNumber());
                        res = planDetailFacade.save(planDetailParent);
                        if (res) {
                            Long parentIdSub = planDetailParent.getId();
                            List<PlanDetailDTO> planDetails = dataDetail.getPlanDetails();
                            if (ListUtil.isNotEmpty(planDetails)) {
                                for (PlanDetailDTO detail : planDetails) {
                                    //再保存子类
                                    PlanDetail planDetailSub = new PlanDetail();
                                    planDetailSub.setParentId(parentIdSub);
                                    planDetailSub.setPlanId(planIdDetail);
                                    planDetailSub.setName(detail.getName());
                                    planDetailSub.setCode(detail.getCode());
                                    planDetailSub.setHospitalId(planDefaultVO.getHospitalId());
                                    planDetailSub.setStatus(detail.getStatus());
                                    planDetailSub.setOrderNo(detail.getOrderNo());
                                    planDetailSub.setNumber(detail.getNumber());
                                    res = planDetailFacade.save(planDetailSub);
                                }
                            }
                        }
                    }
                }
            }
        }
        return res;
    }
}
