package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptCollectionDTO;
import com.diagbot.dto.ConceptCollectionMatchDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.KlConceptCollectionFacade;
import com.diagbot.vo.ConceptCollectionPageVO;
import com.diagbot.vo.ConceptCollectionSaveVO;
import com.diagbot.vo.ConceptCollectionVO;
import com.diagbot.vo.IdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/8/6 14:37
 */
@RestController
@RequestMapping("/kl/conceptCollection")
@Api(value = "术语集合维护相关API", tags = { "术语集合维护相关API" })
@SuppressWarnings("unchecked")
public class KlConceptCollectionController {

    @Autowired
    private KlConceptCollectionFacade klConceptCollectionFacade;

    @ApiOperation(value = "获取术语集合列表[zhaops]",
            notes = "collectionLibName：术语集合名称 <br>" +
                    "collectionLibType： 术语集合类型 <br>" +
                    "conceptLibName： 基础术语名称 <br>" +
                    "collectionRemark： 术语集合说明 <br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<IPage<ConceptCollectionDTO>> getPage(@Valid @RequestBody ConceptCollectionPageVO conceptCollectionPageVO) {
        IPage<ConceptCollectionDTO> data = klConceptCollectionFacade.getConceptCollectionPage(conceptCollectionPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存（新增or修改）[zhaops]",
            notes = "collectionId：术语集合Id，必填 <br>" +
                    "collectionLibType： 术语集合类型，必填 <br>" +
                    "collectionLibName：术语集合名称 <br>" +
                    "relationId： 关系类型 <br>" +
                    "conceptLibType： 术语集合类型 <br>" +
                    "collectionRemark： 术语集合说明 <br>" +
                    "conceptId： 基础术语id，必填 <br>" +
                    "conceptLibType： 基础术语类型，必填 <br>" +
                    "conceptLibName： 基础术语名称 <br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody ConceptCollectionSaveVO conceptCollectionSaveVO) {
        Boolean data = klConceptCollectionFacade.saveOrUpdateRecord(conceptCollectionSaveVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语集合id获取集合相关信息[zhaops]",
            notes = "id： 术语集合id <br>")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    public RespDTO<ConceptCollectionSaveVO> getRecordById(@Valid @RequestBody IdVO idVO) {
        ConceptCollectionSaveVO data = klConceptCollectionFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语集合id删除术语集合关联关系[zhaops]",
            notes = "id： 术语集合id <br>")
    @PostMapping("/deleteById")
    @SysLogger("deleteById")
    public RespDTO<Boolean> deleteById(@Valid @RequestBody IdVO idVO) {
        Boolean data = klConceptCollectionFacade.deleteById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "批量校验标准术语是否存在[zhaops]",
            notes = "conceptLibNames： 基础术语名称 <br>" +
                    "conceptLibType： 基础术语类型 <br>")
    @PostMapping("/collectionMatch")
    @SysLogger("collectionMatch")
    public RespDTO<ConceptCollectionDTO> collectionMatch(@Valid @RequestBody ConceptCollectionVO conceptCollectionVO) {
        ConceptCollectionMatchDTO data = klConceptCollectionFacade.collectionMatch(conceptCollectionVO);
        return RespDTO.onSuc(data);
    }
}