package com.diagbot.util;

import com.github.promeg.pinyinhelper.Pinyin;
import com.github.promeg.pinyinhelper.PinyinMapDict;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 汉字转拼音
 * @author: gaodm
 * @time: 2018/12/14 14:20
 */
public class Cn2SpellUtil {

    private Cn2SpellUtil() {

    }

    //调用类时自动加载字典
    static{
        init();
    }

    //字典设置
    public static void init() {
        Pinyin.init(Pinyin.newConfig()
                .with(new PinyinMapDict() {
                    @Override
                    public Map<String, String[]> mapping() {
                        HashMap<String, String[]> map = new HashMap<String, String[]>();
                        map.put("", new String[] { "HE" });
                        map.put("𬌗", new String[] { "HE" });
                        map.put("", new String[] { "MU" });
                        map.put("𧿹", new String[] { "MU" });
                        return map;
                    }
                }));
    }

    /**
     * 汉字转换位汉语拼音首字母，英文字符不变
     *
     * @param chines 汉字
     * @return 拼音
     */
    public static String converterToFirstSpell(String chines) {
        String pinyinName = "";
        String pinyinRes = Pinyin.toPinyin(chines, "*××*");
        if (StringUtil.isNotBlank(pinyinRes)){
            String[] nameChar = pinyinRes.split("\\*××\\*");
            for (int i = 0; i < nameChar.length; i++) {
                if (StringUtil.isNotBlank(nameChar[i])) {
                    pinyinName += nameChar[i].charAt(0);
                } else {
                    pinyinName += nameChar[i];
                }
            }
        }
        return pinyinName;
    }

    /**
     * 汉字转换位汉语拼音，英文字符不变(大写)
     *
     * @param chines 汉字
     * @return 拼音(大写)
     */
    public static String converterToSpell(String chines) {
        return converterToSpellByCaseType(chines, 1);
    }

    /**
     * 汉字转换位汉语拼音，英文字符不变(小写)
     *
     * @param chines 汉字
     * @return 拼音(小写)
     */
    public static String converterToSpellLow(String chines) {
        return converterToSpellByCaseType(chines, 2);
    }

    /**
     * 汉字转换位汉语拼音，英文字符不变
     *
     * @param chines 汉字
     * @return 拼音
     */
    private static String converterToSpellByCaseType(String chines, Integer caseType) {
        String pinyinName = "";
        if (caseType.equals(2)) {
            pinyinName = Pinyin.toPinyin(chines, "").toLowerCase();
        } else {
            pinyinName = Pinyin.toPinyin(chines, "");
        }
        return pinyinName;
    }

    public static void main(String[] args) {
        System.out.println("================首字母================");
        System.out.println(Cn2SpellUtil.converterToFirstSpell("欢迎来到Java世界"));
        System.out.println(Cn2SpellUtil.converterToFirstSpell("11β-羟化酶缺陷症#11-羟化酶缺陷"));
        System.out.println(Cn2SpellUtil.converterToFirstSpell("-"));
        System.out.println(Cn2SpellUtil.converterToFirstSpell("𬌗-𧿹"));
        System.out.println("================大写全拼================");
        System.out.println(Cn2SpellUtil.converterToSpell("欢迎来到Java世界"));
        System.out.println(Cn2SpellUtil.converterToSpell("11β-羟化酶缺陷症#11-羟化酶缺陷"));
        System.out.println(Cn2SpellUtil.converterToSpell("-"));
        System.out.println(Cn2SpellUtil.converterToSpell("𬌗-𧿹"));
        System.out.println("================小写全拼================");
        System.out.println(Cn2SpellUtil.converterToSpellLow("欢迎来到Java世界"));
        System.out.println(Cn2SpellUtil.converterToSpellLow("11β-羟化酶缺陷症#11-羟化酶缺陷"));
        System.out.println(Cn2SpellUtil.converterToSpellLow("-"));
        System.out.println(Cn2SpellUtil.converterToSpellLow("𬌗-𧿹"));
    }
}
