package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetInquiryDetailSyfDTO;
import com.diagbot.dto.HisInquirySyfDTO;
import com.diagbot.dto.RecordCheckSyfDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SaveInquiryDTO;
import com.diagbot.dto.ScaleDTO;
import com.diagbot.facade.InquiryInfoSyfFacade;
import com.diagbot.vo.GetInquiryDetailVO;
import com.diagbot.vo.GetScaleVO;
import com.diagbot.vo.HisInquirysVO;
import com.diagbot.vo.RecordCheckVO;
import com.diagbot.vo.SaveInquirySyfVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @description: 邵逸夫加入量表定制
 * @author: zhoutg
 * @date: 2021/12/20 16:39
 */
@RestController
@RequestMapping("/inquiryInfoSyf")
@Api(value = "邵逸夫智能预问诊-问诊记录API", tags = { "邵逸夫智能预问诊-问诊记录API" })
@SuppressWarnings("unchecked")
public class InquiryInfoSYFController {

    @Autowired
    InquiryInfoSyfFacade inquiryInfoSyfFacade;

    @ApiOperation(value = "问诊记录保存[by:zhoutg]")
    @PostMapping("/saveInquiry")
    @SysLogger("saveInquiry")
    @Transactional
    public RespDTO<SaveInquiryDTO> saveInquiry(@Valid @RequestBody SaveInquirySyfVO saveInquirySyfVO) {
        return RespDTO.onSuc(inquiryInfoSyfFacade.saveInquiry(saveInquirySyfVO));
    }

    @ApiOperation(value = "历史病历列表(医院对接用)[by:zhoutg]")
    @PostMapping("/hisInquirys")
    @SysLogger("hisInquirys")
    public RespDTO<List<HisInquirySyfDTO>> hisInquirys(@Valid @RequestBody HisInquirysVO hisInquirysVO) {
        return RespDTO.onSuc(inquiryInfoSyfFacade.hisInquirys(hisInquirysVO));
    }

    @ApiOperation(value = "病历详情(医院对接用)[by:zhoutg]")
    @PostMapping("/getInquiryDetail")
    @SysLogger("getInquiryDetail")
    public RespDTO<GetInquiryDetailSyfDTO> getInquiryDetail(@Valid @RequestBody GetInquiryDetailVO getInquiryDetailVO) {
        return RespDTO.onSuc(inquiryInfoSyfFacade.getInquiryDetail(getInquiryDetailVO));
    }

    @ApiOperation(value = "问诊记录检查确认[by:zhoutg]")
    @PostMapping("/recordCheck")
    @SysLogger("recordCheck")
    public RespDTO<RecordCheckSyfDTO> recordCheck(@Valid @RequestBody RecordCheckVO recordCheckVO) {
        return RespDTO.onSuc(inquiryInfoSyfFacade.recordCheck(recordCheckVO));
    }

    @ApiOperation(value = "获取量表结果[by:zhoutg]",
            notes = "inquiryId：病历id<br>" +
                    "scaleName：量表名称")
    @PostMapping("/getScale")
    @SysLogger("getScale")
    public RespDTO<ScaleDTO> getScale(@RequestBody GetScaleVO getScaleVO) {
        return RespDTO.onSuc(inquiryInfoSyfFacade.getScale(getScaleVO));
    }

}
